import tempfile
import requests
from typing import List
from pathlib import Path
from requests import Response
from pypicgo.core.base.result import Result
from pypicgo.core.base.uploader import CommonUploader
from pypicgo.core.models import PluginModel
from pypicgo.core.logger import logger

BASE_DIR = Path(__file__).resolve().parent


class SmmsUploader(CommonUploader):
    name: str = 'SmmsbUploader'
    domain: str = 'https://sm.ms/api/v2'
    secret_token: str

    def __init__(self, secret_token: str,
                 plugins: List[PluginModel],
                 **kwargs):

        self.secret_token =  secret_token
        self.plugins = plugins
        logger.info('load config successfully')    

    @property
    def upload_url(self):
        return f'{self.domain}/upload/'


    def upload(self) -> Result:
        tempfile = BASE_DIR.joinpath(self.file.filename)
        with open(self.file.tempfile.resolve(), 'rb') as f:
            with open(tempfile.resolve(),'wb') as ft:
                ft.write(f.read())
        self.file.tempfile = tempfile
        file = open(self.file.tempfile.resolve(), 'rb')
        files = {
            'smfile': file,
        }
        headers = {'Authorization': f'{self.secret_token}'}

        resp = requests.post(
            url=self.upload_url,
            headers=headers,
            files= files,
        )
        result = self.is_success(resp)
        return result


    def is_success(self, resp: Response) -> Result:
        if resp.status_code == 200:
            origin_resp = resp.json()
            result = origin_resp.get('success')
            if result:
                url = origin_resp['data']['url']
                return Result(
                    tatus=True, 
                    file=self.file, 
                    message=url,
                    remote_url=url,
                    origin_resp=origin_resp
                )
                    
            else:
                code = origin_resp.get('code')
                if code and code == 'image_repeated':
                    url = origin_resp.get('images')
                    return Result(
                        status=True, 
                        file=self.file, 
                        message=url,
                        remote_url=url,
                        origin_resp=origin_resp
                        )
                else:
                    reason = origin_resp.get('message')
                    return Result(status=False, file=self.file, message=reason)

        else:
            origin_resp = dict()
            try:
                origin_resp = resp.json()
            except:
                pass
            reason = origin_resp.get('message', 'upload fail')
            logger.warning(f'upload fail, message:{reason}')
            return Result(status=False, file=self.file, message=reason)



import pyperclip
from pypicgo.core.logger import logger


class ClipBoard:

    @classmethod
    def read(self):
        pass

    @classmethod
    def writer(self, data: str):
        pyperclip.copy(data)
        logger.info(f'writer to ClipBoard: {data}')



from PIL import Image
from pathlib import Path
from pypicgo.core.base.plugin import BeforePlugin
from pypicgo.core.base.file import UploadFile

BASE_DIR = Path(__file__).resolve().parent


class CompressPlugin(BeforePlugin):
    name = 'compress'

    def __init__(self, ratio: float = 1.0, **kwargs):
        super().__init__(**kwargs)
        self.ratio = ratio

    def execute(self, file: UploadFile) -> UploadFile:
        filepath = file.tempfile.resolve()
        filename = file.tempfile.name
        img = Image.open(filepath)
        width, height = img.size
        after_width = int(width * self.ratio)
        after_height = int(height * self.ratio)
        after_filepath = BASE_DIR.joinpath('temp')
        if not after_filepath.exists():
            after_filepath.mkdir(parents=True)
        after_filepath = after_filepath.joinpath(filename)

        img.resize((after_width, after_height), Image.ANTIALIAS)
        img.save(after_filepath.resolve(), quality=95)
        file.tempfile = after_filepath
        return file



from typing import Dict, Optional
from pypicgo.core.base.file import UploadFile


class Result:
    _status: bool
    _file: UploadFile
    _message: str
    origin_resp: Dict

    def __init__(self, status: bool,
                 file: UploadFile,
                 message: str,
                 remote_url:str = None,
                 origin_resp: Dict = dict()
                 ):
        self._status = status
        self._file = file
        self._message = message
        self.remote_url = remote_url
        self.origin_resp = origin_resp

    @property
    def status(self) -> bool:
        return self._status

    @status.setter
    def status(self, status: bool):
        if isinstance(status, bool):
            self._status = status
        else:
            raise ValueError('status must be bool')

    @property
    def file(self) -> UploadFile:
        return self._file

    @file.setter
    def file(self, file: UploadFile):
        if isinstance(file, UploadFile):
            self._file = file
        else:
            raise ValueError('file must be UploadFile')

    @property
    def message(self) -> str:
        return self._message

    @message.setter
    def message(self, message: str):
        if isinstance(message, str):
            self._message = message
        else:
            raise ValueError('message must be str')




from typing import Optional, List
from pydantic import BaseModel
from pypicgo.core.models import PluginModel


class QiNiuUploaderConfig(BaseModel):
    domain: str
    bucket_name: str
    apis: List[str]
    secret_key: str
    access_key: str
    plugins: Optional[List[PluginModel]] = []



import base64
import requests
import json
from typing import List, Optional, Tuple
from requests import Response
from requests_toolbelt import MultipartEncoder
from pypicgo.core.base.uploader import CommonUploader
from pypicgo.core.models import PluginModel
from pypicgo.core.base.result import Result
from .schemas import GiteeUploaderData
from pypicgo.core.logger import logger


class GiteeUploader(CommonUploader):
    name: str = 'giteeUploader'
    domain: str = 'https://gitee.com'
    owner: str
    repo: str
    img_path: str
    access_token: str

    def __init__(self,
                 owner: str,
                 repo: str,
                 img_path: str,
                 access_token: str,
                 plugins: List[PluginModel],
                 **kwargs
                 ):

        self.repo = repo
        self.owner = owner
        self.img_path = img_path
        self.access_token = access_token
        self.plugins = plugins

        logger.info('load config successfully')

    @property
    def base_url(self):
        return f'{self.domain}/api/v5/repos/{self.owner}/{self.repo}'

    def _upload_path(self, filename):
        return f'{self.base_url}/contents/{self.img_path}/{filename}'

    def _get_upload_data(self) -> GiteeUploaderData:
        with open(self.file.tempfile.resolve(), 'rb') as f:
            base64_data = base64.b64encode(f.read())
            filedata = str(base64_data, encoding='utf-8')
        return GiteeUploaderData(
            access_token=self.access_token,
            content=filedata
        )

    def upload(self) -> Result:
        filename = self.file.filename
        data = json.loads(self._get_upload_data().json())
        form_data = MultipartEncoder(data)
        headers = {'Content-Type': form_data.content_type}
        resp = requests.post(
            url=self._upload_path(filename),
            headers=headers,
            data=form_data
        )
        result = self.is_success(resp)

        return result

    def is_success(self, resp: Response) -> Result:
        if resp.status_code == 201:
            origin_resp = resp.json()
            download_url = origin_resp['content']['download_url']
            return Result(
                status=True, 
                file=self.file, 
                message='uload success', 
                remote_url=download_url, 
                origin_resp=origin_resp
                )
        else:
            origin_resp = dict()
            try:
                origin_resp = resp.json()
            except:
                pass
            reason = origin_resp.get('message', 'upload fail')
            logger.warning(f'upload fail, message:{reason}')
            return Result(
                status=False, 
                file=self.file, 
                message=reason,
                )



import yaml
from pathlib import Path
from typing import ClassVar, Dict, List
from pypicgo.core.models import ConfigModel, PluginModel
from pypicgo.core.utils.modules import import_string
from pypicgo.core.exceptions import ConfigException
from pypicgo import BASE_DIR
from pypicgo.core.logger import logger

template = '''
default:
  uploader: gitee
  plugins:
    - module: pypicgo.plugins.rename.ReNamePlugin
      config:
        format: liunx{hash}chenghaiwen{date}-{filename}
    - module: pypicgo.plugins.typora.TyporaPlugin
    ...

uploaders:
  github:
    module: pypicgo.uploaders.github.uploader.GithubUploader
    config:
      domain: https://api.github.com
      owner: xxx
      repo: xxx
      img_path: xxx
      oauth_token: xxx
    plugins:
      - module: pypicgo.plugins.jsdelivr.JsDelivrPlugin
  ....
'''


class Settings:
    CONFIG_DIR: Path = BASE_DIR
    uploader_class: ClassVar
    uploader_config: Dict
    plugins: List[PluginModel]
    uploader_name: str

    def __init__(self, uploader_name=None):
        self.__init_env()
        self.uploader_name = uploader_name
        self.load_config()
        
    def __init_env(self):
        path = self.CONFIG_DIR.joinpath('config.yaml')
        if not path.exists():
            with path.open(mode='w') as f:
                f.write(template)

    def load_config(self):
        config_file = self.CONFIG_DIR.joinpath('config.yaml')
        with open(config_file.resolve(), 'r', encoding='utf-8') as f:
            config = yaml.load(f.read(), Loader=yaml.CFullLoader)
            default = config.get('default', dict())
            common_plugins = default.get('plugins', [])
            try:
                able_uploaders = config['uploaders']
            except KeyError:
                logger.error('No uploader available')
                raise ConfigException('No uploader available')
            
            try:
                default_uploader_name = default['uploader']
            except KeyError:
                logger.error('default uploader is required')
                raise ConfigException('default uploader is required')

            default_uploader = able_uploaders[default_uploader_name]
            if not default_uploader:
                msg = f'not found default uploader {default_uploader_name}'
                logger.error(msg)
                raise ConfigException(msg)
            if self.uploader_name:
                cur_uploader = able_uploaders.get(self.uploader_name, default_uploader)
            else:
                cur_uploader = default_uploader
            uploader_plugins = cur_uploader.get('plugins', [])
            if not uploader_plugins:
                uploader_plugins = []
            plugin_dicts = dict()
            for plugin in common_plugins + uploader_plugins:
                plugin_dicts[plugin['module']] = plugin.get('config')

            plugins = []

            for module, config in plugin_dicts.items():
                plugins.append({
                    'module': module,
                    'config': config
                })
            model = ConfigModel(uploader=cur_uploader, plugins=plugins)
            try:
                module = model.uploader.module
                self.uploader_class = import_string(module)
            except ImportError:
                msg = f'uploader {module} doesnt look like a module path'
                logger.error(msg)
                raise ImportError(msg)
            self.uploader_config = model.uploader.config
            self.plugins = model.plugins



import time
import datetime
import logging
from logging.handlers import TimedRotatingFileHandler
from pypicgo import BASE_DIR

LOG_PATH = BASE_DIR.joinpath('logs')
if not LOG_PATH.exists():
    LOG_PATH.mkdir(parents=True, exist_ok=True)

log_path_info = LOG_PATH.joinpath(f'{time.strftime("%Y-%m-%d")}_info.log')
log_path_error = LOG_PATH.joinpath(f'{time.strftime("%Y-%m-%d")}_info.log')

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger('pypicgo')
logger.setLevel(logging.DEBUG)
info_handler = TimedRotatingFileHandler(log_path_info.resolve(), when='midnight', interval=1, backupCount=7, atTime=datetime.time(0, 0, 0, 0))
info_handler.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(message)s"))

error_handler = TimedRotatingFileHandler(log_path_error, when='midnight', interval=1, backupCount=7, atTime=datetime.time(0, 0, 0, 0))
error_handler.setLevel(logging.ERROR)
error_handler.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(filename)s[:%(lineno)d] - %(message)s"))

logger.addHandler(info_handler)
logger.addHandler(error_handler)
__all__ = ["logger"]



import os
from pathlib import Path

home = os.path.expanduser('~')
HOME = Path(home)
BASE_DIR = HOME.joinpath('.PyPicGo')
if not BASE_DIR.exists():
    BASE_DIR.mkdir(parents=True)

__all__ = ["BASE_DIR"]




from pypicgo.core.base.plugin import AfterPlugin
from pypicgo.core.base.result import Result


class JsDelivrPlugin(AfterPlugin):
    name = 'JsDelivr'

    def execute(self, result: Result):
        remote_url = result.remote_url
        if 'raw.githubusercontent.com' in remote_url:
            img_url = remote_url.replace('raw.githubusercontent.com','cdn.jsdelivr.net/gh')
            args = img_url.split('/')
            front = args[0:6]
            end = args[7:]
            result.remote_url = '/'.join(front + end)



class BaseException(Exception):
    pass


class NotImplementedException(BaseException):

    def __str__(self):
        return 'Mthod is not implemented'


class NotExtendsException(BaseException):

    def __str__(self):
        return 'base class  is not Extends'


class PathNotExistsException(BaseException):
    def __str__(self):
        return 'the path  is not exists'


class IsNotFileException(BaseException):

    def __str__(self):
        return 'the path is not file'


class UploaderTypeException(BaseException):

    def __str__(self):
        return 'not is UploaderPlugin type'


class PluginExecuteException(BaseException):

    def __str__(self):
        return 'Plugin execute exception'


class ConfigException(BaseException):
    
    def __init__(self,message, *args: object) -> None:
        self.message = message
        super().__init__(*args)

    def __str__(self):
        return self.message



from pypicgo.core.base.plugin import AfterPlugin
from pypicgo.core.base.result import Result
from .clipbrd import ClipBoard


class ClipBoardPlugin(AfterPlugin):
    name = 'ClipBoard'

    def execute(self, result: Result):
        if result.status:
            ClipBoard.writer(result.remote_url)



import sys, os
from typing import List
from pypicgo.core.base.plugin import FinallyPlugin
from pypicgo.core.base.result import Result


class TyporaPlugin(FinallyPlugin):
    name = 'Typora'

    def execute(self, results: List[Result]):
        urls = []
        for result in results:
            if result.status:
                urls.append(result.remote_url)
        if len(urls) > 0:
            message = os.linesep.join(urls)
            sys.stdout.write(f'Upload Success:{os.linesep}{message}{os.linesep}')



import argparse

from pypicgo.core.config import Settings
from pypicgo.core.execute import create_uploader
from pypicgo.core.logger import logger


def action():

    parser = argparse.ArgumentParser(
        prog='PyPicGo',
        add_help=True, 
        description='Welcom to PyPicGo'
        )
    parser.add_argument('-n', '--name', type=str, help="uploader name", metavar="github")
    parser.add_argument('-f', '--files', nargs='+', required=True, type=str, help="upload files list", metavar="./img.png")
    args = parser.parse_args()
    uploader_name = args.name
    files = args.files
    settings = Settings(uploader_name=uploader_name)
    uploader = settings.uploader_class
    uploader_config = settings.uploader_config
    plugins = settings.plugins
    with create_uploader(uploader, uploader_config, plugins) as uploader:
        logger.info('upload start')
        for filepath in files:
            logger.info(f'upload file [{filepath}]')
            uploader.do(filepath)
        logger.info('all file has been handled')


if __name__ == '__main__':
    action()



import os
from pathlib import Path
from typing import List, Any

from pypicgo.core.base.plugin import BeforePlugin, AfterPlugin, FinallyPlugin
from pypicgo.core.models import PluginModel
from pypicgo.core.utils.modules import import_string
from pypicgo.core.exceptions import (NotImplementedException, PluginExecuteException)
from pypicgo.core.logger import logger
from pypicgo.core.base.file import UploadFile
from pypicgo.core.base.result import Result


class BaseUploader:
    name = 'base_uploader'


class CommonUploader(BaseUploader):
    type = 'uploader'
    _plugins: List[PluginModel] = []
    _after_plugins: List[Any] = []
    _before_plugins: List[Any] = []
    _final_plugins: List[Any] = []
    file = None
    result = None
    results: List[Result] = []


    def __init__(self, plugins: List[PluginModel], **kwargs):
        self.plugins = plugins

    @property
    def plugins(self):
        return self._plugins

    @plugins.setter
    def plugins(self, plugins: List[PluginModel]):
        if plugins:
            self._plugins = plugins
        else:
            self._plugins = []

    def upload(self) -> Result:
        raise NotImplementedException()

    def get_plugins(self):
        self._after_plugins = []
        self._before_plugins = []
        self._final_plugins = []
        for plugin in self.plugins:
            try:
                module = import_string(plugin.module)
                config = plugin.config if plugin.config else dict()
                if issubclass(module, BeforePlugin):
                    self._before_plugins.append(module(**config))
                elif issubclass(module, AfterPlugin):
                    self._after_plugins.append(module(**config))
                elif issubclass(module, FinallyPlugin):
                    self._final_plugins.append((module(**config)))
            except ImportError:
                logger.warning(f'loader {plugin.module} plugins failed,please check the plugin config')
        logger.info('loader plugins successfully')

    def execute_before_plugins(self) -> UploadFile:
        for plugin in self._before_plugins:
            try:
                self.file = plugin.execute(self.file)
                logger.info(f'plugins [{plugin.name}] execute successfully')
            except PluginExecuteException as e:
                logger.warning(f'plugins [{plugin.name}] execute fail')

        return self.file

    def execute_after_plugins(self, result: Result):
        for plugin in self._after_plugins:
            try:
                plugin.execute(result)
                logger.info(f'plugins [{plugin.name}] execute successfully')
            except PluginExecuteException as e:
                logger.warning(f'plugins [{plugin.name}] execute fail')

    def execute_final_plugins(self, results: List[Result]):
        for plugin in self._final_plugins:
            try:
                plugin.execute(results)
                logger.info(f'plugins [{plugin.name}] execute successfully')
            except PluginExecuteException as e:
                logger.warning(f'plugins [{plugin.name}] execute fail')

    def do(self, filepath: str):
        file = UploadFile(filepath)
        self.file = file
        self.execute_before_plugins()
        try:
            self.result = self.upload()
            if isinstance(self.result, Result):
                self.results.append(self.result)
                if self.result.status is not None:
                    self.execute_after_plugins(self.result)
            else:
                logger.warn(f'upload method of [{self.name}] uploader must return a Result object')
        except Exception as e:
            logger.warn(f'[{self.file.origin_file.resolve()}] upload fail')
    
    def clean_tempfile(self, file: Path):
        if file.origin_file.resolve() != file.tempfile.resolve():
            os.remove(file.tempfile.resolve())

    def final(self):
        self.execute_final_plugins(self.results)
        for result in self.results:
            self.clean_tempfile(result.file)




import requests
import json
import random
import hmac
import time
from hashlib import sha1
from typing import List, Optional, Dict
from requests import Response
from pypicgo.core.base.uploader import CommonUploader
from pypicgo.core.models import PluginModel
from pypicgo.core.base.result import Result
from pypicgo.core.logger import logger
from .utils import urlsafe_base64_encode

_policy_fields = set(
    'callbackUrl',  # 回调URL
    'callbackBody',  # 回调Body
    'callbackHost',  # 回调URL指定的Host
    'callbackBodyType',  # 回调Body的Content-Type
    'callbackFetchKey',  # 回调FetchKey模式开关
    'returnUrl',  # 上传端的303跳转URL
    'returnBody',  # 上传端简单反馈获取的Body
    'endUser',  # 回调时上传端标识
    'saveKey',  # 自定义资源名
    'forceSaveKey',  # saveKey的优先级设置。为 true 时，saveKey不能为空，会忽略客户端指定的key，强制使用saveKey进行文件命名。参数不设置时，默认值为false
    'insertOnly',  # 插入模式开关
    'detectMime',  # MimeType侦测开关
    'mimeLimit',  # MimeType限制
    'fsizeLimit',  # 上传文件大小限制
    'fsizeMin',  # 上传文件最少字节数
    'persistentOps',  # 持久化处理操作
    'persistentNotifyUrl',  # 持久化处理结果通知URL
    'persistentPipeline',  # 持久化处理独享队列
    'deleteAfterDays',  # 文件多少天后自动删除
    'fileType',  # 文件的存储类型，0为普通存储，1为低频存储
    'isPrefixalScope'  # 指定上传文件必须使用的前缀
)


class QiNiuUploader(CommonUploader):
    name: str = 'giteeUploader'
    domain: str = ''
    apis: List = []
    access_key: str = ''
    secret_key: bytes = b''
    bucket_name: str = ''

    def __init__(self, domain: str,
                 bucket_name: str,
                 apis: List[str],
                 access_key: str,
                 secret_key: str,
                 plugins: Optional[List[PluginModel]] = [],
                 **kwargs):
        self.bucket_name = bucket_name
        self.apis = apis
        self.access_key = access_key
        self.domain = domain
        self.secret_key = secret_key
        self.plugins = plugins
        logger.info('init settings successfully')

    @property
    def api(self):
        return random.choice(self.apis)

    def __token(self, data):
        hashed = hmac.new(self.secret_key, data.encode('utf-8'), sha1)
        return urlsafe_base64_encode(hashed.digest())

    def __token_with_data(self, data):
        data = urlsafe_base64_encode(data)
        return '{0}:{1}:{2}'.format(
            self.access_key, self.__token(data), data)

    def get_token(self, bucket: str, policy: Dict, expires: int = 3600):
        if bucket is None or bucket == '':
            raise ValueError('invalid bucket name')
        args = {
            'scope': bucket,
            'deadline': int(time.time()) + expires
        }
        for k, v in policy.items():
            if k in _policy_fields:
                args[k] = v
        data = json.dumps(args, separators=(',', ':'))
        return self.__token_with_data(data)

    def upload(self) -> Result:
        fields = dict()
        filename = self.file.filename
        token = self.get_token(self.bucket_name, {'fileType': 1})
        fields['token'] = token
        fields['key'] = filename
        with open(self.file.tempfile.resolve(), 'rb') as f:
            resp = requests.post(
                url=self.api,
                data=fields,
                files={'file': ('file_name', f, 'application/octet-stream')}
            )

        result = self.is_success(resp=resp)
        return result

    def is_success(self, resp: Response) -> Result:
        if resp.status_code == 200:
            origin_resp = resp.json()
            key = origin_resp['key']
            remote_file = f'{self.domain}{key}'
            return Result(
                status=True, 
                file=self.file,
                message='upload success',
                remote_url=remote_file,
                origin_resp=origin_resp
                )
        else:
            reason = resp.text
            try:
                reason = resp.json().get('error')
            except:
                pass
            logger.warning(f'upload fail, message:{reason}')
            return Result(status=False, file=self.file, message=reason)



from setuptools import setup, find_packages

with open('README.rst', 'r', encoding='utf-8') as fh:
    long_description = fh.read()
setup(
    name='pypicgo',
    version='1.1.0',
    keywords=['python', 'pypicgo'],
    description='A simple & powerful tool for pictures uploading built by python',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT Licence', 
    url='https://github.com/AnsGoo/PyPicGo',
    author='ansgoo',
    author_email='haiven_123@163.com',
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    install_requires=[
        'pydantic>=1.7.4',
        'pyyaml>=5.3.1',
        'requests>=2.22.0',
        'requests-toolbelt>=0.9.1',
        'Pillow>=8.2.0'
    ],
    entry_points={
        'console_scripts': [
            'pypicgo=pypicgo.upload:action'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX :: Linux',
        'Environment :: Console',
        'Environment :: MacOS X',
     ]
)
